package event_tracker

import (
	"testing"

	"github.com/stretchr/testify/require"
)

func TestEventTracker(t *testing.T) {
	et := NewEventTracker()
	e := et.RegisterEvent("x")
	require.Contains(t, et.eventLists, "x")

	ed := et.CloneEventData()
	expectedEvents := map[string][]Event{}
	require.Equal(t, expectedEvents, ed.Events)

	e.EmitEvent(1, 2)
	e.EmitEvent(4, 5)
	e.EmitEvent(1, 2)

	ed = et.CloneEventData()
	expectedEvents = map[string][]Event{
		"x": {
			{
				UserId:    1,
				ProjectId: 2,
			},
			{
				UserId:    4,
				ProjectId: 5,
			},
			{
				UserId:    1,
				ProjectId: 2,
			},
		},
	}
	require.Equal(t, expectedEvents, ed.Events)

	et.Subtract(ed)
	ed = et.CloneEventData()
	require.Empty(t, ed.Events)
}

func TestEventTracker_Subtract(t *testing.T) {
	et := NewEventTracker()
	e := et.RegisterEvent("x")
	require.Contains(t, et.eventLists, "x")

	e.EmitEvent(1, 1)
	e.EmitEvent(2, 2)
	e.EmitEvent(3, 3)
	e.EmitEvent(4, 4)
	e.EmitEvent(5, 5)
	require.Len(t, et.eventLists["x"].events, 5)

	et.Subtract(&EventData{
		Events: map[string][]Event{
			"x": {
				{
					UserId:    1,
					ProjectId: 1,
				},
				{
					UserId:    2,
					ProjectId: 2,
				},
				{
					UserId:    3,
					ProjectId: 3,
				},
			},
		},
	})
	require.Len(t, et.eventLists["x"].events, 2)

	e.EmitEvent(6, 6)

	et.Subtract(&EventData{
		Events: map[string][]Event{
			"x": {
				{
					UserId:    4,
					ProjectId: 4,
				},
				{
					UserId:    5,
					ProjectId: 5,
				},
				{
					UserId:    6,
					ProjectId: 6,
				},
			},
		},
	})
	require.Empty(t, et.eventLists["x"].events)
}
